# This file is a Tcl script to test hugecombo widgets in Tk.  It is
# organized in the standard fashion for Tcl tests.
#
# Copyright (c) 1994 The Regents of the University of California.
# Copyright (c) 1994-1997 Sun Microsystems, Inc.
# Copyright (c) 1998-1999 by Scriptics Corporation.
# All rights reserved.
#
# RCS: @(#) $Id: hugecombo.test,v 1.11 2001/04/03 04:40:50 hobbs Exp $

# ---------------------
#
# packages
#
# ---------------------

package require Tcl 8.4
package require Tk 8.4

package require Hugecombo 1.1
namespace import -force ::hugecombo::hugecombo
package require tcltest 2.1
namespace import -force ::tcltest::test

if {[lsearch [namespace children] ::tcltest] == -1} {
    source [file join [pwd] [file dirname [info script]] defs.tcl]
}

foreach i [winfo children .] {
    destroy $i
}
wm geometry . {}
raise .

proc scroll args {
    global scrollInfo
    set scrollInfo $args
}

# Create additional widget that's used to hold the selection at times.

hugecombo .sel
.sel insert end "This is some sample text"

# Font names

set big -adobe-helvetica-medium-r-normal--24-240-75-75-p-*-iso8859-1
set fixed -adobe-courier-medium-r-normal--12-120-75-75-m-*-iso8859-1

# Create entries in the option database to be sure that geometry options
# like border width have predictable values.

option add *Entry.borderWidth 2
option add *Entry.highlightThickness 2
option add *Entry.font {Helvetica -12}
hugecombo param set e:borderwidth 2
hugecombo param set e:highlightthickness 2

hugecombo .e -bd 2 -relief sunken
pack .e
update

  
  proc bgerror msg { }
  
image create photo img -file dir.gif
set i 1
foreach test {
    {-alternatecolor #ff0000 #ff0000 non-existent {unknown color name "non-existent"}}
    {-background #ff0000 #ff0000 non-existent {unknown color name "non-existent"}}
    {-bd 4 4 badValue {bad screen distance "badValue"}}
    {-bg #ff0000 #ff0000 non-existent {unknown color name "non-existent"}}
    {-borderwidth 1.3 1 badValue {bad screen distance "badValue"}}
    {-compound yes 1 xyzzy {expected boolean value but got "xyzzy"}}
    {-cursor arrow arrow badValue {bad cursor spec "badValue"}}
    {-disabledbackground green green non-existent {unknown color name "non-existent"}}
    {-disabledforeground blue blue non-existent {unknown color name "non-existent"}}
    {-exportselection yes 1 xyzzy {expected boolean value but got "xyzzy"}}
    {-fg #110022 #110022 bogus {unknown color name "bogus"}}
    {-font -Adobe-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-* -Adobe-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-* {} {font "" doesn't exist}}
    {-foreground #110022 #110022 bogus {unknown color name "bogus"}}
    {-height 10 10 3p {expected integer but got "3p"}}
    {-highlightbackground #123456 #123456 ugly {unknown color name "ugly"}}
    {-highlightcolor #123456 #123456 bogus {unknown color name "bogus"}}
    {-highlightthickness 6 6 bogus {bad screen distance "bogus"}}
    {-highlightthickness -2 0 {} {}}
    {-image img img {} {}}
    {-insertbackground #110022 #110022 bogus {unknown color name "bogus"}}
    {-insertborderwidth 1.3 1 2.6x {bad screen distance "2.6x"}}
    {-insertofftime 100 100 3.2 {expected integer but got "3.2"}}
    {-insertontime 100 100 3.2 {expected integer but got "3.2"}}
    {-invalidcommand "any string" "any string" {} {}}
    {-invcmd "any string" "any string" {} {}}
    {-justify right right bogus {bad justification "bogus": must be left, right, or center}}
    {-listvariable var var {} {}}
    {-readonlybackground green green non-existent {unknown color name "non-existent"}}
    {-relief groove groove 1.5 {bad relief "1.5": must be flat, groove, raised, ridge, solid, or sunken}}
    {-rowheight 19.3 19 badValue {bad screen distance "badValue"}}
    {-selectbackground #110022 #110022 bogus {unknown color name "bogus"}}
    {-selectborderwidth 1.3 1 badValue {bad screen distance "badValue"}}
    {-selectforeground #654321 #654321 bogus {unknown color name "bogus"}}
    {-show * * {} {}}
    {-state n normal bogus {bad state "bogus": must be disabled, normal, picklist or readonly}}
    {-takefocus "any string" "any string" {} {}}
    {-textvariable i i {} {}}
    {-values {a b c d} {a b c d} {} {}}
    {-width 402 402 3p {expected integer but got "3p"}}
    {-xscrollcommand {Some command} {Some command} {} {}}
} {
    set name [lindex $test 0]
    test hugecombo-1.$i \
      {configuration options} \
      {
        .e configure $name [lindex $test 1]
        list [lindex [.e configure $name] 4] [.e cget $name]
      } \
      [list [lindex $test 2] [lindex $test 2]]
    incr i
    if {[lindex $test 3] != ""} \
    {
      test hugecombo-1.$i \
        {configuration options} \
        {
          list [catch {.e configure $name [lindex $test 3]} msg] $msg
        } \
        [list 1 [lindex $test 4]]
    }
    .e configure $name [lindex [.e configure $name] 3]
    incr i
}

test hugecombo-2.1 {Tk_EntryCmd procedure} {
    list [catch {hugecombo} msg] $msg
} {1 {unknown option "": should be pathName, option or param}}
test hugecombo-2.2 {Tk_EntryCmd procedure} {
    list [catch {hugecombo gorp} msg] $msg
} {1 {unknown option "gorp": should be pathName, option or param}}
test hugecombo-2.3 {Tk_EntryCmd procedure} {binding_class} {
    catch {destroy .e}
    hugecombo .e
    list [winfo exists .e] [winfo class .e] [info commands .e]
} {1 Entry .e}
test hugecombo-2.4 {Tk_EntryCmd procedure} {
    catch {destroy .e}
    list [catch {hugecombo .e -gorp foo} msg] $msg [winfo exists .e] \
      [info commands .e]
} {1 {unknown option "-gorp"} 0 {}}
test hugecombo-2.5 {Tk_EntryCmd procedure} {
    catch {destroy .e}
    hugecombo .e
} {.e}

catch {destroy .e}
hugecombo .e -font $fixed
pack .e
update

set cx [font measure $fixed a]
set cy [font metrics $fixed -linespace]
set ux [font measure $fixed \u4e4e]

test hugecombo-3.1 {EntryWidgetCmd procedure} {
    list [catch {.e} msg] $msg
} {1 {wrong # args: should be ".e option ?arg arg ...?"}}
test hugecombo-3.2 {EntryWidgetCmd procedure, "bbox" widget command} {
    list [catch {.e bbox} msg] $msg
} {1 {wrong # args: should be ".e bbox index"}}
test hugecombo-3.3 {EntryWidgetCmd procedure, "bbox" widget command} {
    list [catch {.e bbox a b} msg] $msg
} {1 {wrong # args: should be ".e bbox index"}}
test hugecombo-3.4 {EntryWidgetCmd procedure, "bbox" widget command} {
    list [catch {.e bbox bogus} msg] $msg
} {1 {bad hugecombo index "bogus"}}
test hugecombo-3.5 {EntryWidgetCmd procedure, "bbox" widget command} {
    .e delete 0 end
    .e bbox 0
} [list 5 5 0 $cy]
test hugecombo-3.6 {EntryWidgetCmd procedure, "bbox" widget command} {
    # Tcl_UtfAtIndex(): no utf chars

    .e delete 0 end
    .e insert 0 "abc"
    list [.e bbox 3] [.e bbox end]
} [list "[expr 5+2*$cx] 5 $cx $cy" "[expr 5+2*$cx] 5 $cx $cy"]
test hugecombo-3.7 {EntryWidgetCmd procedure, "bbox" widget command} {
    # Tcl_UtfAtIndex(): utf at end
    .e delete 0 end
    .e insert 0 "ab\u4e4e"
    .e bbox end
} "[expr 5+2*$cx] 5 $ux $cy"
test hugecombo-3.8 {EntryWidgetCmd procedure, "bbox" widget command} {
    # Tcl_UtfAtIndex(): utf before index
    .e delete 0 end
    .e insert 0 "ab\u4e4ec"
    .e bbox 3
} "[expr 5+2*$cx+$ux] 5 $cx $cy"
test hugecombo-3.9 {EntryWidgetCmd procedure, "bbox" widget command} {
    # Tcl_UtfAtIndex(): no chars
    .e delete 0 end
    .e bbox end
} "5 5 0 $cy"
test hugecombo-3.10 {EntryWidgetCmd procedure, "bbox" widget command} {
    .e delete 0 end
    .e insert 0 "abcdefghij\u4e4eklmnop"
    list [.e bbox 0] [.e bbox 1] [.e bbox 10] [.e bbox end]
} [list "5 5 $cx $cy" "[expr 5+$cx] 5 $cx $cy" "[expr 5+10*$cx] 5 $ux $cy" "[expr 5+$ux+15*$cx] 5 $cx $cy"]
test hugecombo-3.11 {EntryWidgetCmd procedure, "cget" widget command} {
    list [catch {.e cget} msg] $msg
} {1 {wrong # args: should be ".e cget option"}}
test hugecombo-3.12 {EntryWidgetCmd procedure, "cget" widget command} {
    list [catch {.e cget a b} msg] $msg
} {1 {wrong # args: should be ".e cget option"}}
test hugecombo-3.13 {EntryWidgetCmd procedure, "cget" widget command} {
    list [catch {.e cget -gorp} msg] $msg
} {1 {unknown option "-gorp"}}
test hugecombo-3.14 {EntryWidgetCmd procedure, "cget" widget command} {
    .e configure -bd 4
    .e cget -bd
} {4}
test hugecombo-3.15 {EntryWidgetCmd procedure, "configure" widget command} {options_count} {
    llength [.e configure]
} {36}
test hugecombo-3.16 {EntryWidgetCmd procedure, "configure" widget command} {
    list [catch {.e configure -foo} msg] $msg
} {1 {unknown option "-foo"}}
test hugecombo-3.17 {EntryWidgetCmd procedure, "configure" widget command} {
    .e configure -bd 4
    .e configure -bg #ffffff
    lindex [.e configure -bd] 4
} {4}
test hugecombo-3.18 {EntryWidgetCmd procedure, "delete" widget command} {
    list [catch {.e delete} msg] $msg
} {1 {wrong # args: should be ".e delete firstIndex ?lastIndex?"}}
test hugecombo-3.19 {EntryWidgetCmd procedure, "delete" widget command} {
    list [catch {.e delete a b c} msg] $msg
} {1 {wrong # args: should be ".e delete firstIndex ?lastIndex?"}}
test hugecombo-3.20 {EntryWidgetCmd procedure, "delete" widget command} {
    list [catch {.e delete foo} msg] $msg
} {1 {bad hugecombo index "foo"}}
test hugecombo-3.21 {EntryWidgetCmd procedure, "delete" widget command} {
    list [catch {.e delete 0 bar} msg] $msg
} {1 {bad hugecombo index "bar"}}
test hugecombo-3.22 {EntryWidgetCmd procedure, "delete" widget command} {
    .e delete 0 end
    .e insert end "01234567890"
    .e delete 2 4
    .e get
} {014567890}
test hugecombo-3.23 {EntryWidgetCmd procedure, "delete" widget command} {
    .e delete 0 end
    .e insert end "01234567890"
    .e delete 6
    .e get
} {0123457890}
test hugecombo-3.24 {EntryWidgetCmd procedure, "delete" widget command} {
    # UTF
    set x {}
    .e delete 0 end
    .e insert end "01234\u4e4e67890"
    .e delete 6
    lappend x [.e get]
    .e delete 0 end
    .e insert end "012345\u4e4e7890"
    .e delete 6
    lappend x [.e get]
    .e delete 0 end
    .e insert end "0123456\u4e4e890"
    .e delete 6
    lappend x [.e get]
} [list "01234\u4e4e7890" "0123457890" "012345\u4e4e890"]
test hugecombo-3.25 {EntryWidgetCmd procedure, "delete" widget command} {
    .e delete 0 end
    .e insert end "01234567890"
    .e delete 6 5
    .e get
} {01234567890}
test hugecombo-3.26 {EntryWidgetCmd procedure, "delete" widget command} {
    .e delete 0 end
    .e insert end "01234567890"
    .e configure -state disabled
    .e delete 2 8
    .e configure -state normal
    .e get
} {01234567890}
test hugecombo-3.27 {EntryWidgetCmd procedure, "delete" widget command} {
    .e delete 0 end
    .e insert end "01234567890"
    .e configure -state readonly
    .e delete 2 8
    .e configure -state normal
    .e get
} {01234567890}
test hugecombo-3.27 {EntryWidgetCmd procedure, "get" widget command} {
    list [catch {.e get foo} msg] $msg
} {1 {wrong # args: should be ".e get"}}
test hugecombo-3.28 {EntryWidgetCmd procedure, "icursor" widget command} {
    list [catch {.e icursor} msg] $msg
} {1 {wrong # args: should be ".e icursor pos"}}
test hugecombo-3.29 {EntryWidgetCmd procedure, "icursor" widget command} {
    list [catch {.e icursor foo} msg] $msg
} {1 {bad hugecombo index "foo"}}
test hugecombo-3.30 {EntryWidgetCmd procedure, "icursor" widget command} {
    .e delete 0 end
    .e insert end "01234567890"
    .e icursor 4
    .e index insert
} {4}
test hugecombo-3.31 {EntryWidgetCmd procedure, "index" widget command} {operations_list} {
    list [catch {.e in} msg] $msg
} {1 {ambiguous option "in": must be bbox, cget, configure, delete, get, icursor, index, insert, scan, selection, validate, or xview}}
test hugecombo-3.32 {EntryWidgetCmd procedure, "index" widget command} {
    list [catch {.e index} msg] $msg
} {1 {wrong # args: should be ".e index string"}}
test hugecombo-3.33 {EntryWidgetCmd procedure, "index" widget command} {
    list [catch {.e index foo} msg] $msg
} {1 {bad hugecombo index "foo"}}
test hugecombo-3.34 {EntryWidgetCmd procedure, "index" widget command} {
    list [catch {.e index 0} msg] $msg
} {0 0}
test hugecombo-3.35 {EntryWidgetCmd procedure, "index" widget command} {
    # UTF
    .e delete 0 end
    .e insert 0 abc\u4e4e\u0153def
    list [.e index 3] [.e index 4] [.e index end]
} {3 4 8}
test hugecombo-3.36 {EntryWidgetCmd procedure, "insert" widget command} {
    list [catch {.e insert a} msg] $msg
} {1 {wrong # args: should be ".e insert index text"}}
test hugecombo-3.37 {EntryWidgetCmd procedure, "insert" widget command} {
    list [catch {.e insert a b c} msg] $msg
} {1 {wrong # args: should be ".e insert index text"}}
test hugecombo-3.38 {EntryWidgetCmd procedure, "insert" widget command} {
    list [catch {.e insert foo Text} msg] $msg
} {1 {bad hugecombo index "foo"}}
test hugecombo-3.39 {EntryWidgetCmd procedure, "insert" widget command} {
    .e delete 0 end
    .e insert end "01234567890"
    .e insert 3 xxx
    .e get
} {012xxx34567890}
test hugecombo-3.40 {EntryWidgetCmd procedure, "insert" widget command} {
    .e delete 0 end
    .e insert end "01234567890"
    .e configure -state disabled
    .e insert 3 xxx
    .e configure -state normal
    .e get
} {01234567890}
test hugecombo-3.40 {EntryWidgetCmd procedure, "insert" widget command} {
    .e delete 0 end
    .e insert end "01234567890"
    .e configure -state readonly
    .e insert 3 xxx
    .e configure -state normal
    .e get
} {01234567890}
test hugecombo-3.41 {EntryWidgetCmd procedure, "insert" widget command} {
    list [catch {.e insert a b c} msg] $msg
} {1 {wrong # args: should be ".e insert index text"}}
test hugecombo-3.42 {EntryWidgetCmd procedure, "scan" widget command} {
    list [catch {.e scan a} msg] $msg
} {1 {wrong # args: should be ".e scan mark|dragto x"}}
test hugecombo-3.43 {EntryWidgetCmd procedure, "scan" widget command} {
    list [catch {.e scan a b c} msg] $msg
} {1 {wrong # args: should be ".e scan mark|dragto x"}}
test hugecombo-3.44 {EntryWidgetCmd procedure, "scan" widget command} {
    list [catch {.e scan foobar 20} msg] $msg
} {1 {bad scan option "foobar": must be mark or dragto}}
test hugecombo-3.45 {EntryWidgetCmd procedure, "scan" widget command} {
    list [catch {.e scan mark 20.1} msg] $msg
} {1 {expected integer but got "20.1"}}
# This test is non-portable because character sizes vary.

test hugecombo-3.46 {EntryWidgetCmd procedure, "scan" widget command} {fonts} {
    .e delete 0 end
    update
    .e insert end "This is quite a long string, in fact a "
    .e insert end "very very long string"
    .e scan mark 30
    .e scan dragto 28
    .e index @0
} {2}
test hugecombo-3.47 {EntryWidgetCmd procedure, "select" widget command} {
    list [catch {.e select} msg] $msg
} {1 {wrong # args: should be ".e selection option ?index?"}}
test hugecombo-3.48 {EntryWidgetCmd procedure, "select" widget command} {
    list [catch {.e select foo} msg] $msg
} {1 {bad selection option "foo": must be adjust, clear, from, present, range, or to}}
test hugecombo-3.49 {EntryWidgetCmd procedure, "select clear" widget command} {
    list [catch {.e select clear gorp} msg] $msg
} {1 {wrong # args: should be ".e selection clear"}}
test hugecombo-3.50 {EntryWidgetCmd procedure, "select clear" widget command} {X_selection_behavior} {
    .e delete 0 end
    .e insert end "0123456789"
    .e select from 1
    .e select to 4
    update
    .e select clear
    list [catch {selection get} msg] $msg [selection own]
} {1 {PRIMARY selection doesn't exist or form "STRING" not defined} .e}
test hugecombo-3.51 {EntryWidgetCmd procedure, "selection present" widget command} {
    list [catch {.e selection present foo} msg] $msg
} {1 {wrong # args: should be ".e selection present"}}
test hugecombo-3.52 {EntryWidgetCmd procedure, "selection present" widget command} {
    .e delete 0 end
    .e insert end 0123456789
    .e select from 3
    .e select to 6
    .e selection present
} {1}
test hugecombo-3.53 {EntryWidgetCmd procedure, "selection present" widget command} {
    .e delete 0 end
    .e insert end 0123456789
    .e select from 3
    .e select to 6
    .e configure -exportselection false
    .e selection present
} {1}
.e configure -exportselection true
test hugecombo-3.54 {EntryWidgetCmd procedure, "selection present" widget command} {
    .e delete 0 end
    .e insert end 0123456789
    .e select from 3
    .e select to 6
    .e delete 0 end
    .e selection present
} {0}
test hugecombo-3.55 {EntryWidgetCmd procedure, "selection adjust" widget command} {
    list [catch {.e select adjust x} msg] $msg
} {1 {bad hugecombo index "x"}}
test hugecombo-3.56 {EntryWidgetCmd procedure, "selection adjust" widget command} {
    list [catch {.e select adjust 2 3} msg] $msg
} {1 {wrong # args: should be ".e selection adjust index"}}
test hugecombo-3.57 {EntryWidgetCmd procedure, "selection adjust" widget command} {
    .e delete 0 end
    .e insert end "0123456789"
    .e select from 1
    .e select to 5
    update
    .e select adjust 4
    selection get
} {123}
test hugecombo-3.58 {EntryWidgetCmd procedure, "selection adjust" widget command} {
    .e delete 0 end
    .e insert end "0123456789"
    .e select from 1
    .e select to 5
    update
    .e select adjust 2
    selection get
} {234}
test hugecombo-3.59 {EntryWidgetCmd procedure, "selection from" widget command} {
    list [catch {.e select from 2 3} msg] $msg
} {1 {wrong # args: should be ".e selection from index"}}
test hugecombo-3.60 {EntryWidgetCmd procedure, "selection range" widget command} {
    list [catch {.e select range 2} msg] $msg
} {1 {wrong # args: should be ".e selection range start end"}}
test hugecombo-3.61 {EntryWidgetCmd procedure, "selection range" widget command} {
    list [catch {.e selection range 2 3 4} msg] $msg
} {1 {wrong # args: should be ".e selection range start end"}}
test hugecombo-3.62 {EntryWidgetCmd procedure, "selection range" widget command} {
    .e delete 0 end
    .e insert end 0123456789
    .e select from 1
    .e select to 5
    .e select range 4 4
    list [catch {.e index sel.first} msg] $msg
} {1 {selection isn't in widget .e}}
test hugecombo-3.63 {EntryWidgetCmd procedure, "selection range" widget command} {
    .e delete 0 end
    .e insert end 0123456789
    .e select from 3
    .e select to 7
    .e select range 2 9
    list [.e index sel.first] [.e index sel.last] [.e index anchor]
} {2 9 3}
test hugecombo-3.64 {EntryWidgetCmd procedure, "selection" widget command} {
    .e delete 0 end
    .e insert end 0123456789
    .e selection range 0 end
    .e configure -state disabled
    .e selection range 2 4
    .e configure -state normal
    list [.e index sel.first] [.e index sel.last]
} {0 10}
test hugecombo-3.64 {EntryWidgetCmd procedure, "selection" widget command} {
    .e delete 0 end
    .e insert end 0123456789
    .e selection range 0 end
    .e configure -state readonly
    .e selection range 2 4
    .e configure -state normal
    list [.e index sel.first] [.e index sel.last]
} {2 4}
.e delete 0 end
.e insert end "This is quite a long text string, so long that it "
.e insert end "runs off the end of the window quite a bit."
test hugecombo-3.64 {EntryWidgetCmd procedure, "selection to" widget command} {
    list [catch {.e select to 2 3} msg] $msg
} {1 {wrong # args: should be ".e selection to index"}}
test hugecombo-3.65 {EntryWidgetCmd procedure, "xview" widget command} {
    .e xview 5
    .e xview
} {0.0537634 0.268817}
test hugecombo-3.66 {EntryWidgetCmd procedure, "xview" widget command} {
    list [catch {.e xview gorp} msg] $msg
} {1 {bad hugecombo index "gorp"}}
test hugecombo-3.67 {EntryWidgetCmd procedure, "xview" widget command} {
    .e xview 0
    .e icursor 10
    .e xview insert
    .e xview
} {0.107527 0.322581}
test hugecombo-3.68 {EntryWidgetCmd procedure, "xview" widget command} {
    list [catch {.e xview moveto foo bar} msg] $msg
} {1 {wrong # args: should be ".e xview moveto fraction"}}
test hugecombo-3.69 {EntryWidgetCmd procedure, "xview" widget command} {
    list [catch {.e xview moveto foo} msg] $msg
} {1 {expected floating-point number but got "foo"}}
test hugecombo-3.70 {EntryWidgetCmd procedure, "xview" widget command} {
    .e xview moveto 0.5
    .e xview
} {0.505376 0.72043}
test hugecombo-3.71 {EntryWidgetCmd procedure, "xview" widget command} {
    list [catch {.e xview scroll 24} msg] $msg
} {1 {wrong # args: should be ".e xview scroll number units|pages"}}
test hugecombo-3.72 {EntryWidgetCmd procedure, "xview" widget command} {
    list [catch {.e xview scroll gorp units} msg] $msg
} {1 {expected integer but got "gorp"}}
test hugecombo-3.73 {EntryWidgetCmd procedure, "xview" widget command} {
    .e xview moveto 0
    .e xview scroll 1 pages
    .e xview
} {0.193548 0.408602}
test hugecombo-3.74 {EntryWidgetCmd procedure, "xview" widget command} {
    .e xview moveto .9
    update
    .e xview scroll -2 p
    .e xview
} {0.397849 0.612903}
test hugecombo-3.75 {EntryWidgetCmd procedure, "xview" widget command} {
    .e xview 30
    update
    .e xview scroll 2 units 
    .e index @0
} {32}
test hugecombo-3.76 {EntryWidgetCmd procedure, "xview" widget command} {
    .e xview 30
    update
    .e xview scroll -1 units 
    .e index @0
} {29}
test hugecombo-3.77 {EntryWidgetCmd procedure, "xview" widget command} {
    list [catch {.e xview scroll 23 foobars} msg] $msg
} {1 {bad argument "foobars": must be units or pages}}
test hugecombo-3.78 {EntryWidgetCmd procedure, "xview" widget command} {
    list [catch {.e xview eat 23 hamburgers} msg] $msg
} {1 {unknown option "eat": must be moveto or scroll}}
test hugecombo-3.79 {EntryWidgetCmd procedure, "xview" widget command} {
    .e xview 0
    update
    .e xview -4
    .e index @0
} {0}
test hugecombo-3.80 {EntryWidgetCmd procedure, "xview" widget command} {
    .e xview 300
    .e index @0
} {73}
.e insert 10 \u4e4e
test hugecombo-3.81 {EntryWidgetCmd procedure, "xview" widget command} {
    # UTF
    # If Tcl_NumUtfChars wasn't used, wrong answer would be:
    # 0.106383 0.117021 0.117021

    set x {}
    .e xview moveto .1
    lappend x [lindex [.e xview] 0]
    .e xview moveto .11
    lappend x [lindex [.e xview] 0]
    .e xview moveto .12
    lappend x [lindex [.e xview] 0]
} {0.0957447 0.106383 0.117021}
test hugecombo-3.82 {EntryWidgetCmd procedure} {operations_list} {
    list [catch {.e gorp} msg] $msg
} {1 {bad option "gorp": must be bbox, cget, configure, delete, get, icursor, index, insert, scan, selection, validate, or xview}}

# new operations
# (don't change tests sequence!)

catch {destroy .e}
hugecombo .e
test hugecombo-3.n1 {"component" widget operation} \
{
  .e component button
} \
{.e.b}
test hugecombo-3.n2 {"component" widget operation} \
{
  .e component canvas
} \
{.e.c}
test hugecombo-3.n3 {"component" widget operation} \
{
  .e component entry
} \
{.e.e}
test hugecombo-3.n4 {"component" widget operation} \
{
  .e component hugelist
} \
{.e.t.f.l}
test hugecombo-3.n5 {"component" widget operation} \
{
  .e component listbox
} \
{.e.t.f.l}
test hugecombo-3.n6 {"component" widget operation} \
{
  .e component scrollbar
} \
{.e.t.f.vs}
test hugecombo-3.n7 {"component" widget operation} \
{
  list [catch {.e component other} msg] $msg
} \
{1 {unknown component "other": must be button, canvas, entry, hugelist, listbox or scrollbar}}
.e config -values {a b c d}
test hugecombo-3.n7 {"itemcget" widget operation} \
{
  .e itemcget end-1 -text
} \
{c}
test hugecombo-3.n8 {"itemcget" widget operation} \
{
  .e itemcget end -left
} \
{0}
test hugecombo-3.n9 {"itemcget" widget operation} \
{
  .e itemcget 0 -image
} \
{}
test hugecombo-3.n10 {"itemconfigure" widget operation} \
{
  .e itemconfig end-1 -text new
  .e itemcget end -text
} \
{d}
test hugecombo-3.n11 {"itemconfigure" widget operation} \
{
  .e itemconfig end -left 10
  .e itemcget end -left
} \
{10}
test hugecombo-3.n12 {"itemconfigure" widget operation} \
{
  .e itemconfig 1 -image img
  .e itemcget 1 -image
} \
{img}
test hugecombo-3.n13 {"itemconfigure" widget operation} \
{
  .e itemconfig 1 -img img
  .e itemcget 1 -img
} \
{img}
test hugecombo-3.n14 {"list" widget operation} \
{
  list [catch {.e list} msg] $msg
} \
{1 {wrong # args: should be ".e list command ?args?"}}
test hugecombo-3.n15 {"list" widget operation} \
{
  list [catch {.e list bogus} msg] $msg
} \
{1 {bad option "bogus": must be close, curselection, find, open or select}}
test hugecombo-3.n16 {"list" widget operation} \
{
  .e list open
  winfo ismapped .e.t
} \
{1}
test hugecombo-3.n17 {"list" widget operation} \
{
  .e list close
  winfo ismapped .e.t
} \
{0}
test hugecombo-3.n18 {"list" widget operation} \
{
  .e list find d
  .e list curselection
} \
{3}
test hugecombo-3.n19 {"list" widget operation} \
{
  .e list select end-1
  .e list curselection
} \
{2}
test hugecombo-3.n20 {"load" widget operation} \
{
  .e load
  .e list curselection
} \
{2}
test hugecombo-3.n21 {"load" widget operation} \
{
  .e load end
  .e get
} \
{d}
test hugecombo-3.n21 {"load" widget operation} \
{
  list [catch {.e load bogus} msg] $msg
} \
{1 {bad index "bogus": must be integer or end?-integer?}}
test hugecombo-3.n22 {"send" widget operation} \
{
  list [catch {.e send} msg] $msg
} \
{1 {wrong # args: should be ".e send <keyname>"}}
test hugecombo-3.n23 {"send" widget operation} \
{
  list [catch {.e send bogus} msg] $msg
} \
{1 {unknown key symbol "bogus", should be <Control-End>, <Control-Home>, <Control-Return>, <Down>, <Escape>, <Next>, <Prior>, <Return> or <Up>}}
.e config -values {a b c d e f g h i j}
test hugecombo-3.n24 {"send" widget operation} \
{
  .e send <Control-Home>
  list [winfo ismapped .e.t] [.e list curselection]
} \
{1 0}
test hugecombo-3.n25 {"send" widget operation} \
{
  .e send <Down>
  .e send <Down>
  .e list curselection
} \
{2}
test hugecombo-3.n26 {"send" widget operation} \
{
  .e send <Next>
  .e list curselection
} \
{6}
test hugecombo-3.n27 {"send" widget operation} \
{
  .e send <Up>
  .e list curselection
} \
{5}
test hugecombo-3.n28 {"send" widget operation} \
{
  .e send <Prior>
  .e list curselection
} \
{1}
test hugecombo-3.n29 {"send" widget operation} \
{
  .e send <Control-End>
  .e list curselection
} \
{9}
test hugecombo-3.n30 {"send" widget operation} \
{
  .e send <Return>
  .e get
} \
{j}
test hugecombo-3.n31 {"send" widget operation} \
{
  .e send <Escape>
  list [winfo ismapped .e.t] [.e list curselection] [.e get]
} \
{0 9 j}

# image behavior
# (don't change tests sequence!)

test hugecombo-3.n40 {image behavior} \
{
  .e config -image img
  .e config -compound 0
  list [.e.c cget -width] [.e.c cget -height] [.e cget -img]
} \
{0 0 img}
test hugecombo-3.n41 {image behavior} \
{
  .e itemconfig 0 -img {}
  .e load 0
  list [.e.c cget -width] [.e.c cget -height] [.e cget -img]
} \
{0 0 {}}
test hugecombo-3.n42 {image behavior} \
{
  .e itemconfig 0 -img img
  .e load 0
  .e config -compound 1
  list [.e.c cget -width] [.e.c cget -height] [.e cget -img]
} \
{25 25 img}


# The test below doesn't actually check anything directly, but if run
# with Purify or some other memory-allocation-checking program it will
# ensure that resources get properly freed.

test hugecombo-4.1 {DestroyEntry procedure} {
    catch {destroy .e}
    hugecombo .e -textvariable x -show *
    pack .e
    .e insert end "Sample text"
    update
    destroy .e
} {}

frame .f -width 200 -height 50 -relief raised -bd 2
pack .f -side right
test hugecombo-5.1 {ConfigureEntry procedure, -textvariable} {
    catch {destroy .e}
    set x 12345
    hugecombo .e -textvariable x
    .e get
} {12345}
test hugecombo-5.2 {ConfigureEntry procedure, -textvariable} {
    catch {destroy .e}
    set x 12345
    hugecombo .e -textvariable x
    set y abcde
    .e configure -textvariable y
    set x 54321
    .e get
} {abcde}
test hugecombo-5.3 {ConfigureEntry procedure, -textvariable} {
    catch {destroy .e}
    catch {unset x}
    hugecombo .e
    .e insert 0 "Some text"
    .e configure -textvariable x
    set x
} {Some text}
test hugecombo-5.4 {ConfigureEntry procedure, -textvariable} {
    proc override args {
  global x
  set x 12345
    }
    catch {destroy .e}
    catch {unset x}
    trace variable x w override
    hugecombo .e
    .e insert 0 "Some text"
    .e configure -textvariable x
    set result [list $x [.e get]]
    unset x;  rename override {}
    set result
} {12345 12345}
test hugecombo-5.5 {ConfigureEntry procedure} {
    catch {destroy .e}
    hugecombo .e -exportselection false
    pack .e
    .e insert end "0123456789"
    .sel select from 0
    .sel select to 10
    set x {}
    lappend x [selection get]
    .e select from 1
    .e select to 5
    lappend x [selection get]
    .e configure -exportselection 1
    lappend x [selection get]
    set x
} {{This is so} {This is so} 1234}
test hugecombo-5.6 {ConfigureEntry procedure} {
    catch {destroy .e}
    hugecombo .e
    pack .e
    .e insert end "0123456789"
    .e select from 1
    .e select to 5
    .e configure -exportselection 0
    list [catch {selection get} msg] $msg [.e index sel.first] \
      [.e index sel.last]
} {1 {PRIMARY selection doesn't exist or form "STRING" not defined} 1 5}
test hugecombo-5.7 {ConfigureEntry procedure} {
    catch {destroy .e}
    hugecombo .e -font $fixed -width 4 -xscrollcommand scroll
    pack .e
    .e insert end "01234567890"
    update
    .e configure -width 5
    set scrollInfo
} {0 0.363636}
test hugecombo-5.8 {ConfigureEntry procedure} {fonts} {
    catch {destroy .e}
    hugecombo .e -width 0
    pack .e
    .e insert end "0123"
    update
    .e configure -font $big
    update
    winfo geom .e
} {62x37+0+0}
test hugecombo-5.9 {ConfigureEntry procedure} {fonts} {
    catch {destroy .e}
    hugecombo .e -font $fixed -bd 2 -relief raised
    pack .e
    .e insert end "0123"
    update
    list [.e index @10] [.e index @11] [.e index @12] [.e index @13]
} {0 0 1 1}
test hugecombo-5.10 {ConfigureEntry procedure} {fonts} {
    catch {destroy .e}
    hugecombo .e -font $fixed -bd 2 -relief flat
    pack .e
    .e insert end "0123"
    update
    list [.e index @10] [.e index @11] [.e index @12] [.e index @13]
} {0 0 1 1}
test hugecombo-5.11 {ConfigureEntry procedure} {
    # If "0" in selected font had 0 width, caused divide-by-zero error.

    catch {destroy .e}
    pack [hugecombo .e -font {{open look glyph}}]
    .e scan dragto 30
    update
} {}    

# No tests for DisplayEntry.

test hugecombo-6.1 {EntryComputeGeometry procedure} {fonts} {
    catch {destroy .e}
    hugecombo .e -font $fixed -bd 2 -relief raised -width 20 -highlightthickness 3
    pack .e
    .e insert end 012\t45
    update
    list [.e index @61] [.e index @62]
} {3 4}
test hugecombo-6.2 {EntryComputeGeometry procedure} {fonts} {
    catch {destroy .e}
    hugecombo .e -font $fixed -bd 2 -relief raised -width 20 -justify center \
      -highlightthickness 3
    pack .e
    .e insert end 012\t45
    update
    list [.e index @96] [.e index @97]
} {3 4}
test hugecombo-6.3 {EntryComputeGeometry procedure} {fonts} {
    catch {destroy .e}
    hugecombo .e -font $fixed -bd 2 -relief raised -width 20 -justify right \
      -highlightthickness 3
    pack .e
    .e insert end 012\t45
    update
    list [.e index @131] [.e index @132]
} {3 4}
test hugecombo-6.4 {EntryComputeGeometry procedure} {
    catch {destroy .e}
    hugecombo .e -font $fixed -bd 2 -relief raised -width 5
    pack .e
    .e insert end "01234567890"
    update
    .e xview 6
    .e index @0
} {6}
test hugecombo-6.5 {EntryComputeGeometry procedure} {
    catch {destroy .e}
    hugecombo .e -font $fixed -bd 2 -relief raised -width 5
    pack .e
    .e insert end "01234567890"
    update
    .e xview 7
    .e index @0
} {6}
test hugecombo-6.6 {EntryComputeGeometry procedure} {fonts} {
    catch {destroy .e}
    hugecombo .e -font $fixed -bd 2 -relief raised -width 10
    pack .e
    .e insert end "01234\t67890"
    update
    .e xview 3
    list [.e index @39] [.e index @40]
} {5 6}
test hugecombo-6.7 {EntryComputeGeometry procedure} {fonts} {
    catch {destroy .e}
    hugecombo .e -font $big -bd 3 -relief raised -width 5
    pack .e
    .e insert end "01234567"
    update
    list [winfo reqwidth .e] [winfo reqheight .e]
} {77 39}
test hugecombo-6.8 {EntryComputeGeometry procedure} {fonts} {
    catch {destroy .e}
    hugecombo .e -font $big -bd 3 -relief raised -width 0
    pack .e
    .e insert end "01234567"
    update
    list [winfo reqwidth .e] [winfo reqheight .e]
} {116 39}
test hugecombo-6.9 {EntryComputeGeometry procedure} {fonts} {
    catch {destroy .e}
    hugecombo .e -font $big -bd 3 -relief raised -width 0 -highlightthickness 2
    pack .e
    update
    list [winfo reqwidth .e] [winfo reqheight .e]
} {25 39}
test hugecombo-6.10 {EntryComputeGeometry procedure} {unixOnly fonts} {
    catch {destroy .e}
    hugecombo .e -bd 1 -relief raised -width 0 -show .
    .e insert 0 12345
    pack .e
    update
    set x [winfo reqwidth .e]
    .e configure -show X
    lappend x [winfo reqwidth .e]
    .e configure -show ""
    lappend x [winfo reqwidth .e]
} {23 53 43}
hugecombo param set e:borderwidth 1
test hugecombo-6.11 {EntryComputeGeometry procedure} {pcOnly} {
    catch {destroy .e}
    hugecombo .e -bd 1 -relief raised -width 0 -show . -font {helvetica 12}
    .e insert 0 12345
    pack .e
    update
    set x [winfo reqwidth [.e component entry]]
    .e configure -show X
    update
    lappend x [winfo reqwidth [.e component entry]]
    .e configure -show ""
    update
    lappend x [winfo reqwidth [.e component entry]]
} [list \
    [expr 8+5*[font measure {helvetica 12} .]] \
    [expr 8+5*[font measure {helvetica 12} X]] \
    [expr 8+[font measure {helvetica 12} 12345]]]
hugecombo param set e:borderwidth 2

catch {destroy .e}
hugecombo .e -width 10 -font $fixed -textvariable contents -xscrollcommand scroll
pack .e
focus .e
test hugecombo-7.1 {InsertChars procedure} {
    .e delete 0 end
    .e insert 0 abcde
    .e insert 2 XXX
    update
    list [.e get] $contents $scrollInfo
} {abXXXcde abXXXcde {0 1}}
test hugecombo-7.2 {InsertChars procedure} {
    .e delete 0 end
    .e insert 0 abcde
    .e insert 500 XXX
    update
    list [.e get] $contents $scrollInfo
} {abcdeXXX abcdeXXX {0 1}}
test hugecombo-7.3 {InsertChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789
    .e select from 2
    .e select to 6
    .e insert 2 XXX
    set x "[.e index sel.first] [.e index sel.last]"
    .e select to 8
    lappend x [.e index sel.first] [.e index sel.last]
} {5 9 5 8}
test hugecombo-7.4 {InsertChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789
    .e select from 2
    .e select to 6
    .e insert 3 XXX
    set x "[.e index sel.first] [.e index sel.last]"
    .e select to 8
    lappend x [.e index sel.first] [.e index sel.last]
} {2 9 2 8}
test hugecombo-7.5 {InsertChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789
    .e select from 2
    .e select to 6
    .e insert 5 XXX
    set x "[.e index sel.first] [.e index sel.last]"
    .e select to 8
    lappend x [.e index sel.first] [.e index sel.last]
} {2 9 2 8}
test hugecombo-7.6 {InsertChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789
    .e select from 2
    .e select to 6
    .e insert 6 XXX
    set x "[.e index sel.first] [.e index sel.last]"
    .e select to 5
    lappend x [.e index sel.first] [.e index sel.last]
} {2 6 2 5}
test hugecombo-7.7 {InsertChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789
    .e icursor 4
    .e insert 4 XXX
    .e index insert
} {7}
test hugecombo-7.8 {InsertChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789
    .e icursor 4
    .e insert 5 XXX
    .e index insert
} {4}
test hugecombo-7.9 {InsertChars procedure} {
    .e delete 0 end
    .e insert 0 "This is a very long string"
    update
    .e xview 4
    .e insert 3 XXX
    .e index @0
} {7}
test hugecombo-7.10 {InsertChars procedure} {
    .e delete 0 end
    .e insert 0 "This is a very long string"
    update
    .e xview 4
    .e insert 4 XXX
    .e index @0
} {4}
.e configure -width 0
test hugecombo-7.11 {InsertChars procedure} {fonts} {
    .e delete 0 end
    .e insert 0 "xyzzy"
    update
    .e insert 2 00
    winfo reqwidth .e
} {59}

.e configure -width 10
test hugecombo-8.1 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 abcde
    .e delete 2 4
    update
    list [.e get] $contents $scrollInfo
} {abe abe {0 1}}
test hugecombo-8.2 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 abcde
    .e delete -2 2
    update
    list [.e get] $contents $scrollInfo
} {cde cde {0 1}}
test hugecombo-8.3 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 abcde
    .e delete 3 1000
    update
    list [.e get] $contents $scrollInfo
} {abc abc {0 1}}
test hugecombo-8.4 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789abcde
    .e select from 3
    .e select to 8
    .e delete 1 3
    update
    set x "[.e index sel.first] [.e index sel.last]"
    .e select to 5
    lappend x [.e index sel.first] [.e index sel.last]
} {1 6 1 5}
test hugecombo-8.5 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789abcde
    .e select from 3
    .e select to 8
    .e delete 1 4
    update
    set x "[.e index sel.first] [.e index sel.last]"
    .e select to 4
    lappend x [.e index sel.first] [.e index sel.last]
} {1 5 1 4}
test hugecombo-8.6 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789abcde
    .e select from 3
    .e select to 8
    .e delete 1 7
    update
    set x "[.e index sel.first] [.e index sel.last]"
    .e select to 5
    lappend x [.e index sel.first] [.e index sel.last]
} {1 2 1 5}
test hugecombo-8.7 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789abcde
    .e select from 3
    .e select to 8
    .e delete 1 8
    list [catch {.e index sel.first} msg] $msg
} {1 {selection isn't in widget .e}}
test hugecombo-8.8 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789abcde
    .e select from 3
    .e select to 8
    .e delete 3 7
    update
    set x "[.e index sel.first] [.e index sel.last]"
    .e select to 8
    lappend x [.e index sel.first] [.e index sel.last]
} {3 4 3 8}
test hugecombo-8.9 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789abcde
    .e select from 3
    .e select to 8
    .e delete 3 8
    list [catch {.e index sel.first} msg] $msg
} {1 {selection isn't in widget .e}}
test hugecombo-8.10 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789abcde
    .e select from 8
    .e select to 3
    .e delete 5 8
    update
    set x "[.e index sel.first] [.e index sel.last]"
    .e select to 8
    lappend x [.e index sel.first] [.e index sel.last]
} {3 5 5 8}
test hugecombo-8.11 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789abcde
    .e select from 8
    .e select to 3
    .e delete 8 10
    update
    set x "[.e index sel.first] [.e index sel.last]"
    .e select to 4
    lappend x [.e index sel.first] [.e index sel.last]
} {3 8 4 8}
test hugecombo-8.12 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789abcde
    .e icursor 4
    .e delete 1 4
    .e index insert
} {1}
test hugecombo-8.13 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789abcde
    .e icursor 4
    .e delete 1 5
    .e index insert
} {1}
test hugecombo-8.14 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 0123456789abcde
    .e icursor 4
    .e delete 4 6
    .e index insert
} {4}
test hugecombo-8.15 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 "This is a very long string"
    .e xview 4
    .e delete 1 4
    .e index @0
} {1}
test hugecombo-8.16 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 "This is a very long string"
    .e xview 4
    .e delete 1 5
    .e index @0
} {1}
test hugecombo-8.17 {DeleteChars procedure} {
    .e delete 0 end
    .e insert 0 "This is a very long string"
    .e xview 4
    .e delete 4 6
    .e index @0
} {4}
.e configure -width 0
test hugecombo-8.18 {DeleteChars procedure} {fonts} {
    .e delete 0 end
    .e insert 0 "xyzzy"
    update
    .e delete 2 4
    winfo reqwidth .e
} {31}

test hugecombo-9.1 {EntryValueChanged procedure} {
    catch {destroy .e}
    proc override args {
  global x
  set x 12345
    }
    catch {unset x}
    trace variable x w override
    hugecombo .e -textvariable x
    .e insert 0 foo
    set result [list $x [.e get]]
    unset x; rename override {}
    set result
} {12345 12345}

catch {destroy .e}
hugecombo .e
pack .e
.e configure -width 0
test hugecombo-10.1 {EntrySetValue procedure} {fonts} {
    set x abcde
    set y ab
    .e configure -textvariable x
    update
    .e configure -textvariable y
    update
    list [.e get] [winfo reqwidth .e]
} {ab 24}
test hugecombo-10.2 {EntrySetValue procedure, updating selection} {
    catch {destroy .e}
    hugecombo .e -textvariable x
    .e insert 0 "abcdefghjklmnopqrstu"
    .e selection range 4 10
    set x "a"
    list [catch {.e index sel.first} msg] $msg
} {1 {selection isn't in widget .e}}
test hugecombo-10.3 {EntrySetValue procedure, updating selection} {
    catch {destroy .e}
    hugecombo .e -textvariable x
    .e insert 0 "abcdefghjklmnopqrstu"
    .e selection range 4 10
    set x "abcdefg"
    list [.e index sel.first] [.e index sel.last]
} {4 7}
test hugecombo-10.4 {EntrySetValue procedure, updating selection} {
    catch {destroy .e}
    hugecombo .e -textvariable x
    .e insert 0 "abcdefghjklmnopqrstu"
    .e selection range 4 10
    set x "abcdefghijklmn"
    list [.e index sel.first] [.e index sel.last]
} {4 10}
test hugecombo-10.5 {EntrySetValue procedure, updating display position} {
    catch {destroy .e}
    hugecombo .e -width 10 -font $fixed -textvariable x
    pack .e
    .e insert 0 "abcdefghjklmnopqrstuvwxyz"
    .e xview 10
    update
    set x "abcdefg"
    update
    .e index @0
} {0}
test hugecombo-10.6 {EntrySetValue procedure, updating display position} {
    catch {destroy .e}
    hugecombo .e -width 10 -font $fixed -textvariable x
    pack .e
    .e insert 0 "abcdefghjklmnopqrstuvwxyz"
    .e xview 10
    update
    set x "1234567890123456789012"
    update
    .e index @0
} {10}
test hugecombo-10.7 {EntrySetValue procedure, updating insertion cursor} {
    catch {destroy .e}
    hugecombo .e -width 10 -font $fixed -textvariable x
    pack .e
    .e insert 0 "abcdefghjklmnopqrstuvwxyz"
    .e icursor 5
    set x "123"
    .e index insert
} {3}
test hugecombo-10.8 {EntrySetValue procedure, updating insertion cursor} {
    catch {destroy .e}
    hugecombo .e -width 10 -font $fixed -textvariable x
    pack .e
    .e insert 0 "abcdefghjklmnopqrstuvwxyz"
    .e icursor 5
    set x "123456"
    .e index insert
} {5}

test hugecombo-11.1 {EntryEventProc procedure} {
    catch {destroy .e}
    hugecombo .e
    .e insert 0 abcdefg
    destroy .e
    update
} {}
test hugecombo-11.2 {EntryEventProc procedure} {
    eval destroy [winfo children .]
    hugecombo .e1 -fg #112233
    rename .e1 .e2
    set x {}
    lappend x [winfo children .]
    lappend x [.e2 cget -fg]
    destroy .e1
    lappend x [info command {.e[12]}] [winfo children .]
} {.e1 #112233 {} {}}

test hugecombo-12.1 {EntryCmdDeletedProc procedure} {
    eval destroy [winfo children .]
    button .e1 -text "xyz_123"
    rename .e1 {}
    list [info command {.e[12]}] [winfo children .]
} {{} {}}

catch {destroy .e}
hugecombo .e -font $fixed -width 5 -bd 2 -relief sunken
pack .e
.e insert 0 012345678901234567890
update
.e xview 4
update
test hugecombo-13.1 {GetEntryIndex procedure} {
    .e index end
} {21}
test hugecombo-13.2 {GetEntryIndex procedure} {
    list [catch {.e index abogus} msg] $msg
} {1 {bad hugecombo index "abogus"}}
test hugecombo-13.3 {GetEntryIndex procedure} {
    .e select from 1
    .e select to 6
    .e index anchor
} {1}
test hugecombo-13.4 {GetEntryIndex procedure} {
    .e select from 4
    .e select to 1
    .e index anchor
} {4}
test hugecombo-13.5 {GetEntryIndex procedure} {
    .e select from 3
    .e select to 15
    .e select adjust 4
    .e index anchor
} {15}
test hugecombo-13.6 {GetEntryIndex procedure} {
    list [catch {.e index ebogus} msg] $msg
} {1 {bad hugecombo index "ebogus"}}
test hugecombo-13.7 {GetEntryIndex procedure} {
    .e icursor 2
    .e index insert
} {2}
test hugecombo-13.8 {GetEntryIndex procedure} {
    list [catch {.e index ibogus} msg] $msg
} {1 {bad hugecombo index "ibogus"}}
test hugecombo-13.9 {GetEntryIndex procedure} {
    .e select from 1
    .e select to 6
    list [.e index sel.first] [.e index sel.last]
} {1 6}
selection clear .e
test hugecombo-13.10 {GetEntryIndex procedure} {unixOnly} {
    # On unix, when selection is cleared, hugecombo widget's internal 
    # selection range is reset.

    list [catch {.e index sel.first} msg] $msg
} {1 {selection isn't in widget .e}}
test hugecombo-13.11 {GetEntryIndex procedure} {macOrPc} {
    # On mac and pc, when selection is cleared, hugecombo widget remembers
    # last selected range.  When selection ownership is restored to 
    # hugecombo, the old range will be rehighlighted.

    list [catch {selection get}] [.e index sel.first]
} {1 1}
test hugecombo-13.12 {GetEntryIndex procedure} {unixOnly} {
    list [catch {.e index sbogus} msg] $msg
} {1 {selection isn't in widget .e}}
test hugecombo-13.13 {GetEntryIndex procedure} {macOrPc} {
    list [catch {.e index sbogus} msg] $msg
} {1 {bad hugecombo index "sbogus"}}
test hugecombo-13.14 {GetEntryIndex procedure} {macOrPc} {
    list [catch {selection get}] [catch {.e index sbogus}]
} {1 1}
test hugecombo-13.15 {GetEntryIndex procedure} {
    list [catch {.e index @xyz} msg] $msg
} {1 {bad hugecombo index "@xyz"}}
test hugecombo-13.16 {GetEntryIndex procedure} {fonts} {
    .e index @4
} {4}
test hugecombo-13.17 {GetEntryIndex procedure} {fonts} {
    .e index @11
} {4}
test hugecombo-13.18 {GetEntryIndex procedure} {fonts} {
    .e index @12
} {5}
test hugecombo-13.19 {GetEntryIndex procedure} {fonts} {
    .e index @[expr [winfo width .e] - 6]
} {8}
test hugecombo-13.20 {GetEntryIndex procedure} {fonts} {
    .e index @[expr [winfo width .e] - 5]
} {9}
test hugecombo-13.21 {GetEntryIndex procedure} {
    .e index @1000
} {9}
test hugecombo-13.22 {GetEntryIndex procedure} {
    list [catch {.e index 1xyz} msg] $msg
} {1 {bad hugecombo index "1xyz"}}
test hugecombo-13.23 {GetEntryIndex procedure} {
    .e index -10
} {0}
test hugecombo-13.24 {GetEntryIndex procedure} {
    .e index 12
} {12}
test hugecombo-13.25 {GetEntryIndex procedure} {
    .e index 49
} {21}
test hugecombo-13.26 {GetEntryIndex procedure} {fonts} {
    catch {destroy .e}
    hugecombo .e -show .
    .e insert 0 XXXYZZY
    pack .e
    update
    list [.e index @7] [.e index @8]
} {0 1}

# XXX Still need to write tests for EntryScanTo and EntrySelectTo.

set x {}
for {set i 1} {$i <= 500} {incr i} {
    append x "This is line $i, out of 500\n"
}
test hugecombo-14.1 {EntryFetchSelection procedure} {
    catch {destroy .e}
    hugecombo .e
    .e insert end "This is a test string"
    .e select from 1
    .e select to 18
    selection get
} {his is a test str}
test hugecombo-14.2 {EntryFetchSelection procedure} {
    catch {destroy .e}
    hugecombo .e -show *
    .e insert end "This is a test string"
    .e select from 1
    .e select to 18
    selection get
} {*****************}
test hugecombo-14.3 {EntryFetchSelection procedure} {
    catch {destroy .e}
    hugecombo .e
    .e insert end $x
    .e select from 0
    .e select to end
    string compare [selection get] $x
} 0

test hugecombo-15.1 {EntryLostSelection} {
    catch {destroy .e}
    hugecombo .e
    .e insert 0 "Text"
    .e select from 0
    .e select to 4
    set result [selection get]
    selection clear
    .e select from 0
    .e select to 4
    lappend result [selection get]
} {Text Text}

# No tests for EventuallyRedraw.

catch {destroy .e}
hugecombo .e -width 10 -xscrollcommand scroll
pack .e
update

test hugecombo-16.1 {EntryVisibleRange procedure} {fonts} {
    .e delete 0 end
    .e insert 0 .............................
    .e xview
} {0 0.827586}
test hugecombo-15.2 {EntryVisibleRange procedure} {unixOnly fonts} {
    .e configure -show X
    .e delete 0 end
    .e insert 0 .............................
    .e xview
} {0 0.275862}
test hugecombo-15.3 {EntryVisibleRange procedure} {pcOnly} {
    .e configure -show .
    .e delete 0 end
    .e insert 0 XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
    .e xview
} {0 0.827586}
.e configure -show ""
test hugecombo-15.4 {EntryVisibleRange procedure} {
    .e delete 0 end
    .e xview
} {0 1}

catch {destroy .e}
hugecombo .e -width 10 -xscrollcommand scroll -font $fixed
pack .e
update
test hugecombo-17.1 {EntryUpdateScrollbar procedure} {
    .e delete 0 end
    .e insert 0 123
    update
    set scrollInfo
} {0 1}
test hugecombo-17.2 {EntryUpdateScrollbar procedure} {
    .e delete 0 end
    .e insert 0 0123456789abcdef
    .e xview 3
    update
    set scrollInfo
} {0.1875 0.8125}
test hugecombo-17.3 {EntryUpdateScrollbar procedure} {
    .e delete 0 end
    .e insert 0 abcdefghijklmnopqrs
    .e xview 6
    update
    set scrollInfo
} {0.315789 0.842105}
test hugecombo-17.4 {EntryUpdateScrollbar procedure} {bgerror_result} {
    destroy .e
    proc bgerror msg {
  global x
  set x $msg
    }
    hugecombo .e -width 5 -xscrollcommand thisisnotacommand
    pack .e
    update
    rename bgerror {}
    list $x $errorInfo
} {{invalid command name "thisisnotacommand"} {invalid command name "thisisnotacommand"
    while executing
"thisisnotacommand 0 1"
    (horizontal scrolling command executed by .e)}}

set l [interp hidden]
eval destroy [winfo children .]

test hugecombo-18.1 {Entry widget vs hiding} {
    destroy .e
    hugecombo .e
    interp hide {} .e
    destroy .e
    list [winfo children .] [interp hidden]
} [list {} $l]    

##
## Entry widget VALIDATION tests
##

destroy .e
catch {unset ::e}
catch {unset ::vVals}
hugecombo .e -validate all \
  -validatecommand [list doval %W %d %i %P %s %S %v %V] \
  -invalidcommand bell \
  -textvariable ::e \
  -background red -foreground white
pack .e
proc doval {W d i P s S v V} {
    set ::vVals [list [winfo parent $W] $d $i $P $s $S $v $V]
    return 1
}

# The validation tests build each one upon the previous, so cascading
# failures aren't good
#
test hugecombo-19.1 {hugecombo widget validation} {
    .e insert 0 a
    set ::vVals
} {.e 1 0 a {} a all key}
test hugecombo-19.2 {hugecombo widget validation} {
    .e insert 1 b
    set ::vVals
} {.e 1 1 ab a b all key}
test hugecombo-19.3 {hugecombo widget validation} {
    .e insert end c
    set ::vVals
} {.e 1 2 abc ab c all key}
test hugecombo-19.4 {hugecombo widget validation} {
    .e insert 1 123
    list $::vVals $::e
} {{.e 1 1 a123bc abc 123 all key} a123bc}
test hugecombo-19.5 {hugecombo widget validation} {
    .e delete 2
    set ::vVals
} {.e 0 2 a13bc a123bc 2 all key}
test hugecombo-19.6 {hugecombo widget validation} {
    .e configure -validate key
    .e delete 1 3
    set ::vVals
} {.e 0 1 abc a13bc 13 key key}
test hugecombo-19.7 {hugecombo widget validation} {
    set ::vVals {}
    .e configure -validate focus
    .e insert end d
    set ::vVals
} {}
test hugecombo-19.8 {hugecombo widget validation} {
    focus -force .e
    # update necessary to process FocusIn event
    update
    set ::vVals
} {.e -1 -1 abcd abcd {} focus focusin}
test hugecombo-19.9 {hugecombo widget validation} {
    focus -force .
    # update necessary to process FocusOut event
    update
    set ::vVals
} {.e -1 -1 abcd abcd {} focus focusout}
.e configure -validate all
test hugecombo-19.10 {hugecombo widget validation} {
    focus -force .e
    # update necessary to process FocusIn event
    update
    set ::vVals
} {.e -1 -1 abcd abcd {} all focusin}
test hugecombo-19.11 {hugecombo widget validation} {
    focus -force .
    # update necessary to process FocusOut event
    update
    set ::vVals
} {.e -1 -1 abcd abcd {} all focusout}
.e configure -validate focusin
test hugecombo-19.12 {hugecombo widget validation} {
    focus -force .e
    # update necessary to process FocusIn event
    update
    set ::vVals
} {.e -1 -1 abcd abcd {} focusin focusin}
test hugecombo-19.13 {hugecombo widget validation} {
    set ::vVals {}
    focus -force .
    # update necessary to process FocusOut event
    update
    set ::vVals
} {}
.e configure -validate focuso
test hugecombo-19.14 {hugecombo widget validation} {
    focus -force .e
    # update necessary to process FocusIn event
    update
    set ::vVals
} {}
test hugecombo-19.15 {hugecombo widget validation} {
    focus -force .
    # update necessary to process FocusOut event
    update
    set ::vVals
} {.e -1 -1 abcd abcd {} focusout focusout}
test hugecombo-19.16 {hugecombo widget validation} {
    list [.e validate] $::vVals
} {1 {.e -1 -1 abcd abcd {} all forced}}
test hugecombo-19.17 {hugecombo widget validation} {
    set ::e newdata
    list [.e cget -validate] $::vVals
} {focusout {.e -1 -1 newdata abcd {} focusout forced}}

proc doval {W d i P s S v V} {
    set ::vVals [list [winfo parent $W] $d $i $P $s $S $v $V]
    return 0
}

test hugecombo-19.18 {hugecombo widget validation} {
    .e configure -validate all
    set ::e nextdata
    list [.e cget -validate] $::vVals
} {none {.e -1 -1 nextdata newdata {} all forced}}

proc doval {W d i P s S v V} {
    set ::vVals [list [winfo parent $W] $d $i $P $s $S $v $V]
    set ::e mydata
    return 1
}

## This sets validate to none because it shows that we prevent a possible
## loop condition in the validation, when the hugecombo textvar is also set
test hugecombo-19.19 {hugecombo widget validation} {
    .e configure -validate all
    .e validate
    list [.e cget -validate] [.e get] $::vVals
} {none mydata {.e -1 -1 nextdata nextdata {} all forced}}

## This leaves validate alone because we trigger validation through the
## textvar (a write trace), and the write during validation triggers
## nothing (by definition of avoiding loops on var traces).  This is
## one of those "dangerous" conditions where the user will have a
## different value in the hugecombo widget shown as is in the textvar.
test hugecombo-19.20 {hugecombo widget validation} {
    .e configure -validate all
    set ::e testdata
    list [.e cget -validate] [.e get] $::e $::vVals
} {all testdata mydata {.e -1 -1 testdata mydata {} all forced}}

destroy .e
catch {unset ::e ::vVals}

##
## End validation tests
##

test hugecombo-20.1 {widget deletion while active} {
    destroy .e
    hugecombo .e -validate all \
      -validatecommand { destroy %W ; return 1 } \
      -invalidcommand bell
    update
    .e insert 0 abc
    winfo exists .e
} 0
test hugecombo-20.2 {widget deletion while active} {
    destroy .e
    hugecombo .e -validate all \
      -validatecommand { return 0 } \
      -invalidcommand { destroy %W }
    .e insert 0 abc
    winfo exists .e
} 0
test hugecombo-20.3 {widget deletion while active} {
    destroy .e
    hugecombo .e -validate all \
      -validatecommand { rename .e {} ; return 1 }
    .e insert 0 abc
    winfo exists .e
} 0
test hugecombo-20.4 {widget deletion while active} {
    destroy .e
    hugecombo .e -validate all \
      -validatecommand { return 0 } \
      -invalidcommand { rename .e {} }
    .e insert 0 abc
    winfo exists .e
} 0
test hugecombo-20.5 {widget deletion while active} {
    destroy .e
    hugecombo .e -validatecommand { destroy .e ; return 0 }
    .e validate
    winfo exists .e
} 0
test hugecombo-20.6 {widget deletion while active} {
    destroy .e
    pack [hugecombo .e]
    update
    .e config -xscrollcommand { destroy .e }
    update idle
    winfo exists .e
} 0

# XXX Still need to write tests for EntryBlinkProc, EntryFocusProc,
# and EntryTextVarProc.

option clear

# cleanup
::tcltest::cleanupTests
#return
